﻿using Kivii.Finances.Addons;
using Kivii.Finances.Entities;
using Kivii.Office.OpenXml.Excel;
using Kivii.Web;
using Kivii;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Njhg.Finances.Addons
{
    [Api(Description = "南京海关网银流水读取")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentReadExcelNjhgReceipt : RestfulExecution<Payment>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            (Request.Files.Count() < 1 || Request.Files == null).ThrowIfTrue("未找到excel文件!");
            var httpFile = Request.Files[0];
            ExcelPackage package = new ExcelPackage(Request.Files[0].InputStream);
            (package.Workbook.Worksheets.Count == 0).ThrowIfTrue("未找到Sheet!");

            var rtns = new ReadExcelResponse<Payment>();
            rtns.Results = new List<Payment>();
            try
            {
                var sheet = package.Workbook.Worksheets[1];

                var ePayeeName = sheet.Cells[1, 4].Value;
                var payeeName = "南京海关工业产品检测中心";
                if (ePayeeName != null)
                {
                    payeeName = ePayeeName.ToString();
                }
                var payeeAccount = "4132200001819900011640";
                var ePayeeAccount = sheet.Cells[1, 2].Value;
                if (ePayeeAccount != null)
                {
                    payeeAccount = ePayeeAccount.ToString();
                }
                var end = sheet.Dimension.End.Row;
                for (int i = 8; i <= end; i++)
                {
                    #region 格式化Excel中每行数据,必填项若不符合验证条件就抛出异常
                    var eOperateDate = sheet.Cells[i, 2].Value;
                    if (eOperateDate == null)
                    {
                        throw new Exception($"Excel第{i}行第2列不可为空,请传入交易日期!");
                    }
                    string operate = string.Empty;
                    var eOperateTime = sheet.Cells[i, 3].Value;
                    if (eOperateTime == null)
                    {
                        operate = eOperateDate.ToString();
                        //throw new Exception($"Excel第{i}行第8列不可为空,请传入交易时间!");
                    }
                    else
                    {
                        operate = $"{eOperateDate} {eOperateTime}";
                    }
                    if (!DateTime.TryParse(operate, out DateTime operateTime))
                    {
                        throw new Exception($"Excel第{i}行第2列交易日期格式不正确!");
                    }
                    var eAmount = sheet.Cells[i, 5].Value;
                    if (eAmount == null) continue;
                    if (!Decimal.TryParse(eAmount.ToString(), out var amount)) continue;
                    //{
                    //    throw new Exception($"Excel第{i}行第5列金额格式不正确!");
                    //}
                    if (amount <= 0)
                    {
                        throw new Exception($"Excel第{i}行第5列金额不能小于等于零!");
                    }
                    var eSerialNumber = sheet.Cells[i, 9].Value;
                    if (eSerialNumber == null)
                    {
                        throw new Exception($"Excel第{i}行第9列不可为空,请传入不重复的流水号!");
                    }
                    var serialNumber = $"{operateTime:yyyyMMddHHmmss}{eSerialNumber}";

                    var ePayerName = sheet.Cells[i, 7].Value;
                    if (ePayerName == null)
                    {
                        throw new Exception($"Excel第{i}行第7列不可为空,请传入付款单位!");
                    }
                    var payerName = ePayerName.ToString();
                    var type = PaymentType.Bank;
                    var currency = CurrencyUnit.CNY;
                    
                    var eBank = sheet.Cells[i, 8].Value;
                    var bank = eBank == null ? string.Empty : eBank.ToString();
                    var eBankNumber = sheet.Cells[i, 6].Value;
                    var bankNumber = eBankNumber == null ? string.Empty : eBankNumber.ToString();
                    var eSummary = sheet.Cells[i, 11].Value;
                    var summary = eSummary == null ? string.Empty : eSummary.ToString();
                    var eRemark = sheet.Cells[i, 10].Value;
                    var remark = eRemark == null ? string.Empty : eRemark.ToString();
                    #endregion

                    var payment = new Payment();
                    payment.Metadata = new Dictionary<string, string>();
                    payment.SerialNumber = serialNumber;
                    payment.Type = type;
                    payment.Currency = currency;
                    payment.OperateTime = operateTime;
                    payment.Amount = amount;
                    payment.PayeeName= payerName;
                    payment.PayeeAccountSerialNumber = payeeAccount;
                    payment.PayerName = payerName;
                    payment.PayerAccountName = bank;
                    payment.PayerAccountSerialNumber = bankNumber;
                    payment.Summary = summary;
                    payment.Remark = remark;
                    rtns.Results.Add(payment);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            rtns.Total = rtns.Results.Count;
            rtns.success = true;
            return rtns;
        }
    }
}
